==> libcsys-2.7.0.tar.gz: libcsys-v2.7.0/csys/storageinfo.cpp <==
/*
    * This file is a part of LibCSys.
    * Library for managing drive and getting system resource information in real time for CuboCore Application Suite
	* Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, vsit http://www.gnu.org/licenses/.
    *
*/

#include "storageinfo.h"
#define DBUS_SERVICE "org.freedesktop.UDisks2"
#define DBUS_PATH "/org/freedesktop/UDisks2/block_devices/"

/*
	*
	* StorageManager - Singleton class that manages all the storage volumes
	*
*/

StorageManager* StorageManager::mStorageManager = nullptr;

StorageManager* StorageManager::instance() {

	if ( not mStorageManager )
		mStorageManager = new StorageManager();

	return mStorageManager;
};

StorageDevices StorageManager::devices() {

	return mDevices;
};

StorageBlocks StorageManager::blocks() {

	return mBlocks;
};

StorageBlocks StorageManager::validBlocks() {

	return mValidBlocks;
};

StorageManager::StorageManager() : QObject() {

	devWatch = new QFileSystemWatcher();
	if ( not devWatch->addPath( "/dev/disk/by-id/" ) )
		qCritical() << "Failed to add /dev/disk/by-id/ to QFileSystemWatcher. Auto device detection disabled";

	connect( devWatch, SIGNAL( directoryChanged( QString ) ), this, SLOT( detectChanges() ) );

	rescanStorage();
};

void StorageManager::rescanStorage() {

	mDevices.clear();
	mBlocks.clear();
	mValidBlocks.clear();

	mDeviceNames.clear();
	mBlockNames.clear();

	QDir disks( "/dev/disk/by-id/", QString(), QDir::Name | QDir::IgnoreCase, QDir::System | QDir::Files );
	Q_FOREACH( QFileInfo info, disks.entryInfoList() ) {

		/* ID of the block device, line sda1, sr0, etc... */
		QString blockID = QFileInfo( info.symLinkTarget() ).baseName();

		/* if blockID is empty, do not proceed */
		if ( blockID.isEmpty() )
			continue;

		/* StorageBlock */
		StorageBlock block( blockID );

		/* DriveID: TSSTcorp...., WDC-WD1..., JetFlash-Trans.... etc... */
		QString driveID = QFileInfo( block.drive() ).baseName();

		/* If driveID is empty, do not proceed */
		if ( driveID.isEmpty() )
			continue;

		// /* If the device is already added, move on */
		if ( mDeviceNames.contains( driveID ) )
			continue;

		mDeviceNames << driveID;
		mDevices << StorageDevice( driveID );
	}

	QDir dev( "/dev/block/", QString(), QDir::Name | QDir::IgnoreCase, QDir::System | QDir::Files );
	Q_FOREACH( QFileInfo info, dev.entryInfoList() ) {

		/* Device ID, like sda, sdb1, sr0, etc... */
		QString devID = QFileInfo( info.symLinkTarget() ).baseName();

		/* devID is empty, do not proceed */
		if ( devID.isEmpty() )
			continue;

		/* If this block is already added, move on */
		if ( mBlockNames.contains( devID ) )
			continue;

		/* Add the device ID to a list */
		mBlockNames << devID;

		StorageBlock block( devID );
		mBlocks << block;

		/* Valid FS for a block device; not excluded */
		if ( block.fileSystem().count() and block.fileSystem() != "swap" )
			mValidBlocks << block;
	}
};

void StorageManager::detectChanges() {

	QSet<QString> oldDevs = mBlockNames.toSet();

	rescanStorage();
	QSet<QString> newDevs = mBlockNames.toSet();

	Q_FOREACH( QString dev, QSet<QString>( newDevs ).subtract( oldDevs ) )
		emit deviceAdded( "/org/freedesktop/UDisks2/block_devices/" + dev );

	Q_FOREACH( QString dev, QSet<QString>( oldDevs ).subtract( newDevs ) )
		emit deviceRemoved( "/org/freedesktop/UDisks2/block_devices/" + dev );
};

/*
	*
	* StorageDevice - Class to access the physical devices
	*
*/

StorageDevice::StorageDevice() {

	mLabel = QString();
	mPath = QString();
	mId = QString();
	mIsRemovable = false;
	mIsOptical = false;
	mSize = 0;
	mRotationRate = 0;
	mSeat = QString();
};

StorageDevice::StorageDevice( QString driveID ) {

	iface = new QDBusInterface( DBUS_SERVICE, "/org/freedesktop/UDisks2/drives/" + driveID, QString( "%1.Drive" ).arg( DBUS_SERVICE ), QDBusConnection::systemBus() );

	mLabel = property( "Vendor" ).toString() + " " + property( "Model" ).toString();
	mPath = iface->path();
	mId = property( "Id" ).toString();
	mIsRemovable = property( "Removable" ).toBool();
	mIsOptical = ( property( "MediaCompatibility" ).toStringList().filter( "optical" ).count() > 0 );
	mSize = property( "Size" ).toULongLong();
	mRotationRate = property( "RotationRate" ).toInt();
	mSeat = property( "Seat" ).toString();

	readPartitions();
};

StorageBlocks StorageDevice::partitions() {

	return mParts;
};

StorageBlocks StorageDevice::validPartitions() {

	return mValidParts;
};

QString StorageDevice::label() {

	return mLabel;
};

QString StorageDevice::path() {

	return mPath;
};

QString StorageDevice::id() {

	return mId;
};

bool StorageDevice::isRemovable() {

	return mIsRemovable;
};

bool StorageDevice::isOptical() {

	return mIsOptical;
};

quint64 StorageDevice::size() {

	return mSize;
};

int StorageDevice::rotationRate() {

	return mRotationRate;
};

QString StorageDevice::seat() {

	return mSeat;
};

QVariant StorageDevice::property( QString key ) {

	if ( not iface )
		return QVariant();

	return iface->property( key.toLatin1().constData() );
};

void StorageDevice::readPartitions() {

	if ( not iface )
		return;

	mParts.clear();
	mPartNames.clear();

	QDir dev( "/dev/block/", QString(), QDir::Name | QDir::IgnoreCase, QDir::System | QDir::Files );
	Q_FOREACH( QFileInfo info, dev.entryInfoList() ) {
		/* Device ID, like sda, sdb1, sr0, etc... */
		QString devID = QFileInfo( info.symLinkTarget() ).baseName();

		if ( mPartNames.contains( devID ) )
			continue;

		StorageBlock block( devID );
		if ( block.drive() == iface->path() ) {
			mParts << block;
			mPartNames << devID;

			/* Valid FS for a block device; not excluded */
			if ( block.fileSystem().count() and block.fileSystem() != "swap" )
				mValidParts << block;
		}
	}
};

StorageBlock::StorageBlock() {

	mLabel = QString();
	mPath = QString();
	mDevice = QString();
	mDrive = QString();
	mMountPoint = QString();
	mFileSystem = QString();
	mIsOptical = false;
	mIsRemovable = false;
	mAvailableSize = 0;
	mTotalSize = 1;
};

StorageBlock::StorageBlock( QString id ) {

	mPath = DBUS_PATH + id;
	mDevice = "/dev/" + id;
	getMountPoint();

	QDBusConnection sysBus = QDBusConnection::systemBus();
	QDBusInterface blockIFace( DBUS_SERVICE, mPath, QString( "%1.Block" ).arg( DBUS_SERVICE ), QDBusConnection::systemBus() );
	QDBusInterface partIFace( DBUS_SERVICE, mPath, QString( "%1.Partition" ).arg( DBUS_SERVICE ), QDBusConnection::systemBus() );
	QDBusInterface fsIFace( DBUS_SERVICE, mPath, QString( "%1.FileSystem" ).arg( DBUS_SERVICE ), QDBusConnection::systemBus() );

	QString name = blockIFace.property( "IdLabel" ).toString().simplified();

	/* If we don't get the name from IdLabel */
	if ( not name.count() )
		name = QFileInfo( mMountPoint ).baseName();

	/* If the device is not mounted */
	if ( not name.count() )
		name = id;

	mLabel = QString( name );

	mDrive = blockIFace.property( "Drive" ).value<QDBusObjectPath>().path();
	mFileSystem = blockIFace.property( "IdType" ).toString();

	QDBusInterface driveIFace( DBUS_SERVICE, mDrive, QString( "%1.Drive" ).arg( DBUS_SERVICE ), QDBusConnection::systemBus() );

	QStringList compat = driveIFace.property( "MediaCompatibility" ).toStringList();
	mIsOptical = compat.filter( "optical_" ).count() > 0;

	mIsRemovable = driveIFace.property( "Removable" ).toBool();
	mTotalSize = partIFace.property( "Size" ).toULongLong();

	if ( not mMountPoint.isEmpty() )
		mAvailableSize = QStorageInfo( mMountPoint ).bytesAvailable();
};

void StorageBlock::getMountPoint() {

	QFile dev( "/etc/mtab" );
	dev.open( QFile::ReadOnly );

	QStringList mounts = QString::fromLocal8Bit( dev.readAll() ).split( "\n", QString::SkipEmptyParts );
	Q_FOREACH( QString mount, mounts ) {
		if ( mount.startsWith( mDevice + " " ) ) {
			mMountPoint = mount.split( QRegExp( "\\s" ), QString::SkipEmptyParts )[ 1 ];
			break;
		}
	}
};

QString StorageBlock::label() {

	return mLabel;
};

QString StorageBlock::path() {

	return mPath;
};

QString StorageBlock::device() {

	return mDevice;
};

QString StorageBlock::drive() {

	return mDrive;
};

QString StorageBlock::mountPoint() {

	return mMountPoint;
};

QString StorageBlock::fileSystem() {

	return mFileSystem;
};

bool StorageBlock::isOptical() {

	return mIsOptical;
};

bool StorageBlock::isRemovable() {

	return mIsRemovable;
};

quint64 StorageBlock::totalSize() {

	return mTotalSize;
};

quint64 StorageBlock::availableSize() {

	return mAvailableSize;
};

QVariant StorageBlock::property( QString interface, QString key ) {

	QDBusInterface iface( DBUS_SERVICE, mPath, QString( DBUS_SERVICE ) + "." + interface, QDBusConnection::systemBus() );
	if ( not iface.isValid() ) {
		qCritical() << QObject::tr( "Failed D-Bus connection." );
		return QVariant();
	}

	return iface.property( key.toLocal8Bit().constData() );
};

bool StorageBlock::mount() {

		if ( not mIsOptical ) {
			QDBusInterface filesystem( DBUS_SERVICE, mPath, QString( "%1.Filesystem" ).arg( DBUS_SERVICE ), QDBusConnection::systemBus() );
			if ( !filesystem.isValid() ) {
				qCritical() << QObject::tr( "Failed D-Bus connection." );
				return false;
			}

			QVariantMap options;
			if ( mFileSystem == "vfat" )
				options.insert( "options", "flush" );

			QDBusReply<QString> mountpoint = filesystem.call( "Mount", options );
			QString errStr( mountpoint.error().message() );
			if ( not errStr.isEmpty() ) {
				qCritical() << "[QDBus Error]" << errStr;
				return false;
			}

			return true;
		}

		/* Mount this optical deivce */
		else {
			/* UDisks2 DBus API cannot mount an optical device (bug); use udisks command */
			QProcess proc;
			proc.start( QString( "udisksctl" ), QStringList() << "mount" << "-b" << mDevice );
			proc.waitForFinished();

			return ( proc.exitCode() > 0 ? false : true );
		}
};

bool StorageBlock::unmount() {

	if ( not mIsOptical ) {
		QDBusInterface filesystem( DBUS_SERVICE, mPath, QString( "%1.Filesystem" ).arg( DBUS_SERVICE ), QDBusConnection::systemBus() );
		if ( !filesystem.isValid() ) {
			qCritical() << QObject::tr( "Failed D-Bus connection." );
			return false;
		}

		QDBusMessage reply = filesystem.call( "Unmount", QVariantMap() );

		QString errStr( reply.errorMessage() );
		if ( not errStr.isEmpty() ) {
			qCritical() << errStr;
			return false;
		}

		return true;
	}

	/* Unmount this optical device */
	else {

		/* UDisks2 DBus API cannot unmount an optical device (bug); use udisks command */
		QProcess proc;
		proc.start( QString( "udisksctl" ), QStringList() << "unmount" << "-b" << mDevice );
		proc.waitForFinished();

		return ( proc.exitCode() > 0 ? false : true );
	}
};

==> libcsys-2.8.0.tar.gz: libcsys-v2.8.0/csys/storageinfo.cpp <==
/*
    * This file is a part of LibCSys.
    * Library for managing drive and getting system resource information in real time for CuboCore Application Suite
	* Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, vsit http://www.gnu.org/licenses/.
    *
*/

#include "storageinfo.h"
#define DBUS_SERVICE "org.freedesktop.UDisks2"
#define DBUS_PATH "/org/freedesktop/UDisks2/block_devices/"

/*
	*
	* StorageManager - Singleton class that manages all the storage volumes
	*
*/

StorageManager* StorageManager::mStorageManager = nullptr;

StorageManager* StorageManager::instance() {

	if ( not mStorageManager )
		mStorageManager = new StorageManager();

	return mStorageManager;
};

StorageDevices StorageManager::devices() {

	return mDevices;
};

StorageBlocks StorageManager::blocks() {

	return mBlocks;
};

StorageBlocks StorageManager::validBlocks() {

	return mValidBlocks;
};

StorageManager::StorageManager() : QObject() {

	devWatch = new QFileSystemWatcher();
	if ( not devWatch->addPath( "/dev/disk/by-id/" ) )
		qCritical() << "Failed to add /dev/disk/by-id/ to QFileSystemWatcher. Auto device detection disabled";

	connect( devWatch, SIGNAL( directoryChanged( QString ) ), this, SLOT( detectChanges() ) );

	rescanStorage();
};

void StorageManager::rescanStorage() {

	mDevices.clear();
	mBlocks.clear();
	mValidBlocks.clear();

	mDeviceNames.clear();
	mBlockNames.clear();

	QDBusInterface iface( DBUS_SERVICE, "/org/freedesktop/UDisks2/block_devices", "org.freedesktop.DBus.Introspectable", QDBusConnection::systemBus() );
	QDBusReply<QString> reply = iface.call( "Introspect" );

	if ( not reply.isValid() ) {
		return;
	}

	QXmlStreamReader xml( reply.value() );
	QStringList response;

	while ( !xml.atEnd() ) {
		xml.readNext();

		if ( xml.tokenType() == QXmlStreamReader::StartElement && xml.name().toString() == "node" ) {
			QString blockID = xml.attributes().value( "name" ).toString();

			if ( blockID.isEmpty() )
				continue;

			/* StorageBlock */
			StorageBlock block( blockID );

			/* DriveID: TSSTcorp...., WDC-WD1..., JetFlash-Trans.... etc... */
			QString driveID = QFileInfo( block.drive() ).baseName();

			/* If this block is already added, move on */
			if ( not mBlockNames.contains( blockID ) ) {
				/* Add the device ID to a list */
				mBlockNames << blockID;
				mBlocks << block;

				/* Valid FS for a block device; not excluded */
				if ( block.fileSystem().count() and block.fileSystem() != "swap" )
					mValidBlocks << block;
			}

			/* If driveID is empty, do not proceed */
			if ( not driveID.isEmpty() and not mDeviceNames.contains( driveID ) ) {
				mDeviceNames << driveID;
				mDevices << StorageDevice( driveID );
			}
		}
	}
};

void StorageManager::detectChanges() {

	QSet<QString> oldDevs = mBlockNames.toSet();

	rescanStorage();
	QSet<QString> newDevs = mBlockNames.toSet();

	Q_FOREACH( QString dev, QSet<QString>( newDevs ).subtract( oldDevs ) )
		emit deviceAdded( "/org/freedesktop/UDisks2/block_devices/" + dev );

	Q_FOREACH( QString dev, QSet<QString>( oldDevs ).subtract( newDevs ) )
		emit deviceRemoved( "/org/freedesktop/UDisks2/block_devices/" + dev );
};

/*
	*
	* StorageDevice - Class to access the physical devices
	*
*/

StorageDevice::StorageDevice() {

	mLabel = QString();
	mPath = QString();
	mId = QString();
	mIsRemovable = false;
	mIsOptical = false;
	mSize = 0;
	mRotationRate = 0;
	mSeat = QString();
};

StorageDevice::StorageDevice( QString driveID ) : mDriveId( driveID ) {

	iface = new QDBusInterface( DBUS_SERVICE, "/org/freedesktop/UDisks2/drives/" + driveID, QString( "%1.Drive" ).arg( DBUS_SERVICE ), QDBusConnection::systemBus() );

	mLabel = property( "Vendor" ).toString() + " " + property( "Model" ).toString();
	mPath = iface->path();
	mId = property( "Id" ).toString();
	mIsRemovable = property( "Removable" ).toBool();
	mIsOptical = ( property( "MediaCompatibility" ).toStringList().filter( "optical" ).count() > 0 );
	mSize = property( "Size" ).toULongLong();
	mRotationRate = property( "RotationRate" ).toInt();
	mSeat = property( "Seat" ).toString();
};

StorageBlocks StorageDevice::partitions() {

	if ( mParts.isEmpty() )
		readPartitions();

	return mParts;
};

StorageBlocks StorageDevice::validPartitions() {

	if ( mValidParts.isEmpty() )
		readPartitions();

	return mValidParts;
};

QString StorageDevice::label() {

	return mLabel;
};

QString StorageDevice::path() {

	return mPath;
};

QString StorageDevice::id() {

	return mId;
};

bool StorageDevice::isRemovable() {

	return mIsRemovable;
};

bool StorageDevice::isOptical() {

	return mIsOptical;
};

quint64 StorageDevice::size() {

	return mSize;
};

int StorageDevice::rotationRate() {

	return mRotationRate;
};

QString StorageDevice::seat() {

	return mSeat;
};

QVariant StorageDevice::property( QString key ) {

	if ( not iface )
		return QVariant();

	return iface->property( key.toLatin1().constData() );
};

void StorageDevice::readPartitions() {

	mParts.clear();
	mPartNames.clear();
	mValidParts.clear();

	Q_FOREACH( StorageBlock block, StorageManager::instance()->blocks() ) {
		if ( QFileInfo( block.drive() ).baseName() == mDriveId ) {
			if ( not mPartNames.contains( block.path() ) ) {
				mParts << block;
				mPartNames << block.path();
				if ( block.fileSystem().count() and block.fileSystem() != "swap" )
					mValidParts << block;
			}
		}
	}
};

StorageBlock::StorageBlock() {

	mLabel = QString();
	mPath = QString();
	mDevice = QString();
	mDrive = QString();
	mMountPoint = QString();
	mFileSystem = QString();
	mIsOptical = false;
	mIsRemovable = false;
	mAvailableSize = 0;
	mTotalSize = 1;
};

StorageBlock::StorageBlock( QString id ) {

	mPath = DBUS_PATH + id;
	mDevice = "/dev/" + id;
	getMountPoint();

	QDBusConnection sysBus = QDBusConnection::systemBus();
	QDBusInterface blockIFace( DBUS_SERVICE, mPath, QString( "%1.Block" ).arg( DBUS_SERVICE ), QDBusConnection::systemBus() );
	QDBusInterface partIFace( DBUS_SERVICE, mPath, QString( "%1.Partition" ).arg( DBUS_SERVICE ), QDBusConnection::systemBus() );
	QDBusInterface fsIFace( DBUS_SERVICE, mPath, QString( "%1.FileSystem" ).arg( DBUS_SERVICE ), QDBusConnection::systemBus() );

	QString name = blockIFace.property( "IdLabel" ).toString().simplified();

	/* If we don't get the name from IdLabel */
	if ( not name.count() )
		name = QFileInfo( mMountPoint ).baseName();

	/* If the device is not mounted */
	if ( not name.count() )
		name = id;

	mLabel = QString( name );

	mDrive = blockIFace.property( "Drive" ).value<QDBusObjectPath>().path();
	mFileSystem = blockIFace.property( "IdType" ).toString();

	QDBusInterface driveIFace( DBUS_SERVICE, mDrive, QString( "%1.Drive" ).arg( DBUS_SERVICE ), QDBusConnection::systemBus() );

	QStringList compat = driveIFace.property( "MediaCompatibility" ).toStringList();
	mIsOptical = compat.filter( "optical_" ).count() > 0;

	mIsRemovable = driveIFace.property( "Removable" ).toBool();
	mTotalSize = partIFace.property( "Size" ).toULongLong();

	if ( not mMountPoint.isEmpty() )
		mAvailableSize = QStorageInfo( mMountPoint ).bytesAvailable();
};

void StorageBlock::getMountPoint() {

	QFile dev( "/etc/mtab" );
	dev.open( QFile::ReadOnly );

	QStringList mounts = QString::fromLocal8Bit( dev.readAll() ).split( "\n", QString::SkipEmptyParts );
	Q_FOREACH( QString mount, mounts ) {
		if ( mount.startsWith( mDevice + " " ) ) {
			mMountPoint = mount.split( QRegExp( "\\s" ), QString::SkipEmptyParts )[ 1 ];
			break;
		}
	}
};

QString StorageBlock::label() {

	return mLabel;
};

QString StorageBlock::path() {

	return mPath;
};

QString StorageBlock::device() {

	return mDevice;
};

QString StorageBlock::drive() {

	return mDrive;
};

QString StorageBlock::mountPoint() {

	return mMountPoint;
};

QString StorageBlock::fileSystem() {

	return mFileSystem;
};

bool StorageBlock::isOptical() {

	return mIsOptical;
};

bool StorageBlock::isRemovable() {

	return mIsRemovable;
};

quint64 StorageBlock::totalSize() {

	return mTotalSize;
};

quint64 StorageBlock::availableSize() {

	return mAvailableSize;
};

QVariant StorageBlock::property( QString interface, QString key ) {

	QDBusInterface iface( DBUS_SERVICE, mPath, QString( DBUS_SERVICE ) + "." + interface, QDBusConnection::systemBus() );
	if ( not iface.isValid() ) {
		qCritical() << QObject::tr( "Failed D-Bus connection." );
		return QVariant();
	}

	return iface.property( key.toLocal8Bit().constData() );
};

bool StorageBlock::mount() {

		if ( not mIsOptical ) {
			QDBusInterface filesystem( DBUS_SERVICE, mPath, QString( "%1.Filesystem" ).arg( DBUS_SERVICE ), QDBusConnection::systemBus() );
			if ( !filesystem.isValid() ) {
				qCritical() << QObject::tr( "Failed D-Bus connection." );
				return false;
			}

			QVariantMap options;
			if ( mFileSystem == "vfat" )
				options.insert( "options", "flush" );

			QDBusReply<QString> mountpoint = filesystem.call( "Mount", options );
			QString errStr( mountpoint.error().message() );
			if ( not errStr.isEmpty() ) {
				qCritical() << "[QDBus Error]" << errStr;
				return false;
			}

			return true;
		}

		/* Mount this optical deivce */
		else {
			/* UDisks2 DBus API cannot mount an optical device (bug); use udisks command */
			QProcess proc;
			proc.start( QString( "udisksctl" ), QStringList() << "mount" << "-b" << mDevice );
			proc.waitForFinished();

			return ( proc.exitCode() > 0 ? false : true );
		}
};

bool StorageBlock::unmount() {

	if ( not mIsOptical ) {
		QDBusInterface filesystem( DBUS_SERVICE, mPath, QString( "%1.Filesystem" ).arg( DBUS_SERVICE ), QDBusConnection::systemBus() );
		if ( !filesystem.isValid() ) {
			qCritical() << QObject::tr( "Failed D-Bus connection." );
			return false;
		}

		QDBusMessage reply = filesystem.call( "Unmount", QVariantMap() );

		QString errStr( reply.errorMessage() );
		if ( not errStr.isEmpty() ) {
			qCritical() << errStr;
			return false;
		}

		return true;
	}

	/* Unmount this optical device */
	else {

		/* UDisks2 DBus API cannot unmount an optical device (bug); use udisks command */
		QProcess proc;
		proc.start( QString( "udisksctl" ), QStringList() << "unmount" << "-b" << mDevice );
		proc.waitForFinished();

		return ( proc.exitCode() > 0 ? false : true );
	}
};
